--[[ oDEPS
	Hello Mr. Localizer / Bug fixer,
	
	If you want to localize this add on, feel free to send me a diff/patch on troeks@gmail.com.
	Don't commit however, since that puts me out of control regarding changes, and it's very
	common that the add on breaks with syntax errors or incorrectly added externals.
	
	I'm very tired of this happening, so the consequences of this will be that I removed the
	add on from the SVN as people think they can tinker with everything without asking.
	
	This is the final warning.
	
	(This also yields for every other type of commit that effects this add on.)
]]

local S = AceLibrary("Surface-1.0")
local C = AceLibrary("CandyBar-2.0")
local G = getfenv(0)
local W = {
	["Slice and Dice"] = 2,
	["Eviscerate"] = 1,
	["Expose Armor"] = 1,
	["Rupture"] = 1,
	["Kidney Shot"] = 1,
}
local Defaults = {
	min = 1.5,
	max = 600,
	growth = true,
	texture = "Glaze",
}

S:Register("Otravi", "Interface\\AddOns\\oDEPS\\textures\\otravi")
S:Register("Perl", "Interface\\AddOns\\oDEPS\\textures\\perl")
S:Register("Smooth", "Interface\\AddOns\\oDEPS\\textures\\smooth")
S:Register("Glaze", "Interface\\AddOns\\oDEPS\\textures\\glaze")
S:Register("Striped", "Interface\\AddOns\\oDEPS\\textures\\striped")
S:Register("BantoBar", "Interface\\AddOns\\oDEPS\\textures\\BantoBar")

local addon = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDebug-2.0", "AceDB-2.0", "AceHook-2.1", "AceConsole-2.0")

local function candyCount(text, value)
	if((not text) or (value == nil) or (value < 0)) then return end
	local bar = C.var.handlers[text]
	if(not bar) then return end
	
	if(bar.time >= value)then
		bar.elapsed = value
		C:Update(text)
	else
		C:StopCandyBar(text)
	end
end

local function candyStart(id, dur, name, icon, reg)
	if(not reg) then
		C:RegisterCandyBar(id, dur, name, icon, "Green", "Orange", "Red")
		C:RegisterCandyBarWithGroup(id, "oDEPS")
	end
	C:SetCandyBarTexture(id, S:Fetch(addon.db.profile.texture))
	C:StartCandyBar(id, not reg)
end

function addon:OnInitialize()
	self.Options = {
		type="group",
		args = {
			lock = {
				name = "Lock", type = "toggle",
				desc = "Lock the frame",
				get = function() return self.db.profile.locked end,
				set = function(v) self:toggleMove(v) end,
			},
			growth = {
				name = "Bars", type = "toggle",
				message = "%s will grow: [%s]",
				map = { [false] = "Down", [true] = "Up" },
				desc = "Bar growth direction",
				get = function() return self.db.profile.growth end,
				set = function(v)
					self.db.profile.growth = v
					C:SetCandyBarGroupGrowth("oDEPS", self.db.profile.growth)
				end,
			},
			texture = {
				name = "Texture", type = 'text',
				desc = "Toggle the texture.",
				get = function() return self.db.profile.texture end,
				set = function(v)
					self.db.profile.texture = v
				end,
				validate = S:List()
			},
		}
	}
	self:RegisterDB("oDEPSDB")
	self:RegisterDefaults('profile', Defaults)
	self:RegisterChatCommand({"/oDEPS"}, self.Options)

	self:CreateFrame()

	C:RegisterCandyBarGroup("oDEPS")
	C:SetCandyBarGroupPoint("oDEPS", "TOP", self.frame, "TOP", 0, -12)
end

function addon:OnEnable()
	self:checkSnD()
	self:regTempCB()
	self:ScheduleRepeatingEvent("oDEPS_WeaponEnchants", function()
		local mh, _, mhQ, oh, _, ohQ = GetWeaponEnchantInfo()
		local texture, time, dur, run

		if(mh) then
			_, _, dur, run = C:CandyBarStatus("oDEPS MH")
			if(not run and mhQ > 0) then
				candyStart("oDEPS MH", nil, nil, nil, 1)
				C:PauseCandyBar("oDEPS MH")
			elseif(run and mhQ == 0) then
				C:StopCandyBar("oDEPS MH")
			end

			candyCount("oDEPS MH", mhQ)
		else
			C:StopCandyBar("oDEPS MH")
		end
	
		if(oh) then
			_, _, _dur, run = C:CandyBarStatus("oDEPS OH")
			if(not run and ohQ > 0) then
				candyStart("oDEPS OH", nil, nil, nil, 1)
				C:PauseCandyBar("oDEPS OH")
			elseif(run and ohQ == 0) then
				C:StopCandyBar("oDEPS OH")
			end

			candyCount("oDEPS OH", ohQ)
		else
			C:StopCandyBar("oDEPS OH")
		end
	end, 2) -- Two seconds should be way more then needed.

	self:RegisterEvent("UNIT_SPELLCAST_SENT", function(unit, spell)
		if(unit ~= "plyer") then
			if(W[spell]) then
				local id = "oDEPS Find Weakness"
				candyStart(id, 10, "Find Weakness", "Interface\\Icons\\Ability_Rogue_FindWeakness")

				if(W[spell] == 2) then
					id = "oDEPS "..spell
					local t = self.snd * (6 + (3 * GetComboPoints()))
					candyStart(id, t, spell, "Interface\\Icons\\Ability_Rogue_SliceDice")
				end
			end
		end
	end)

	self.Options.args.texture.validate = S:List()
	self:RegisterEvent("Surface_Registered", function()
		self.Options.args.texture.validate = S:List()
	end)
end

function addon:OnDisable()
end

function addon:checkSnD()
	local _, _, _, _, rank = GetTalentInfo(1,6)

	self.snd = (rank == 0 and 1) or (rank == 1 and 1.15) or (rank == 2 and 1.3) or (rank == 3 and 1.45)
end

function addon:regTempCB()
	C:RegisterCandyBar("oDEPS MH", 120, "MH", GetInventoryItemTexture("player", 16))
	C:RegisterCandyBar("oDEPS OH", 120, "OH", GetInventoryItemTexture("player", 17))

	C:RegisterCandyBarWithGroup("oDEPS MH", "oDEPS")
	C:RegisterCandyBarWithGroup("oDEPS OH", "oDEPS")

	C:SetCandyBarTimeFormat("oDEPS MH", function(a1, a2)
		local _, _, cu = C:CandyBarStatus(a2)
		return string.format("%d", cu)
	end, "oDEPS MH")
	C:SetCandyBarTimeFormat("oDEPS OH", function(a1, a2)
		local _, _, cu = C:CandyBarStatus(a2)
		return string.format("%d", cu)
	end, "oDEPS OH")
end

function addon:CreateFrame()
	self.frame = CreateFrame("Frame", "oDEPSFrame", UIParent)
	self.frame:EnableMouse(true)
	self.frame:SetMovable(true)
	self.frame:SetWidth(236)
	self.frame:SetHeight(56)
	
	self.frame.title = self.frame:CreateFontString(nil, "ARTWORK","TextStatusBarText")
	self.frame.title:SetText("oDEPS")
	self.frame.title:SetPoint("TOP",0, 0)
	
	self.frame:SetBackdrop( { 
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
		edgeFile = "", tile = true, tileSize = 16, edgeSize = 16, 
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
	})
	self.frame:SetBackdropColor(0, 0, 0, .6)
	
	self.frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	self.frame:SetScript("OnMouseDown",function()
		if ( arg1 == "LeftButton" ) then
			this:StartMoving()
		end
	end)
	self.frame:SetScript("OnMouseUp",function()
		if ( arg1 == "LeftButton" ) then
			this:StopMovingOrSizing()
			self:savePosition()
		end
	end)

	if(self.db.profile.locked) then
		self.frame:Hide()
	end

	self:updatePositions()
end

function addon:toggleMove(a1)
	self.db.profile.locked = a1
	if(a1) then
		self.frame:Hide()
	else
		self.frame:Show()
	end
end

function addon:updatePositions()
	if(self.db.profile.Pos) then
		local x, y = strsplit(" ", self.db.profile.Pos)
		local s = self.frame:GetEffectiveScale()
		
		self.frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x/s, y/s)
	else
		self.frame:ClearAllPoints()
		self.frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end
end

function addon:savePosition()
	local f = self.frame
	local x,y = f:GetLeft(), f:GetTop()
	local s = f:GetEffectiveScale()
	
	x,y = x*s,y*s
	
	self.db.profile.Pos = strjoin(" ", x, y)
end

G["oDEPS"] = addon
